#!/usr/bin/env python
#
# Browser bookmarks menu as a GNOME panel applet.
#
# (C) 2004 Nigel Tao.
# Licensed under the GNU GPL.
#
# TO DO:
# + global keyboard binding
# + persistent history
# + customizable engines

import gconf
import gnome
import gnome.applet
import gnome.ui
import gobject
import gtk
import os
import sys

from gettext import gettext as _



APP_NAME = "deskbar-applet"
VERSION = "0.2"

# Over-ride STOCK_FIND so that our find button has no text.
# See the PyGTK FAQ 9.5.
gtk.stock_add([(gtk.STOCK_FIND, None, 0, 0, None)])

gray = gtk.gdk.color_parse("gray")

# engines are represented as a 3-tuple of (accelerator, name, URL)
engines = [
	("f", "Search for Files",      "gnome-search-tool --path=$HOME --named=\"*\" --start"),
	("d", "Dictionary",            "gnome-dictionary \"*\""),
	(None, None, None),
	("t", "AltaVista",             "http://www.altavista.com/web/results?q=*"),
	("o", "Amazon (A9)",           "http://www.a9.com/*"),
	("b", "CDDB",                  "http://www.gracenote.com/music/search.html?f=artist&f=disc&q=*"),
	("r", "Creative Commons",      "http://search.creativecommons.org/index.jsp?q=*"),
	("e", "Ebay",                  "http://search.ebay.com/*"),
	("g", "Google",                "http://www.google.com/search?q=*"),
	("i", "IMDB",                  "http://www.imdb.com/find?q=*"),
	("m", "MSN Search",            "http://search.msn.com/results.aspx?q=*"),
	("p", "Open Directory (DMOZ)", "http://search.dmoz.org/cgi-bin/search?search=*"),
	("q", "Stock Quotes (Yahoo)",  "http://finance.yahoo.com/q?s=*"),
	("w", "Wikipedia",             "http://en.wikipedia.org/wiki/*"),
	("y", "Yahoo",                 "http://search.yahoo.com/search?p=*"),
	]

default_engine = ("g", "Google", "http://www.google.com/search?q=*")


gnome_datadir = gnome.program_init(APP_NAME, VERSION).get_property(gnome.PARAM_GNOME_DATADIR)

completions = gtk.ListStore(gobject.TYPE_STRING)
completions.set_sort_column_id(0, gtk.SORT_ASCENDING)

def gconf_get_entry_width(gconf_prefix):
	w = gconf.client_get_default().get_int(gconf_prefix + "/entry_width")
	if w <= 0:
		w = 150
	return w


def gconf_set_entry_width(gconf_prefix, width):
	gconf.client_get_default().set_int(gconf_prefix + "/entry_width", int(width))


def gconf_save_history(history):
	gconf.client_get_default().set_list("/apps/deskbar-applet/history", gconf.VALUE_STRING, history)


def gconf_load_history():
	global completions
	history = gconf.client_get_default().get_list("/apps/deskbar-applet/history", gconf.VALUE_STRING)
	for h in history:
		completions.append([h])


def list_store_to_list(list_store):
	result = []
	for i in range(len(list_store)):
		result.append(list_store[i][0])
	return result


def add_to_completions(c):
	# remove duplicate entries
	try:
		index = list_store_to_list(completions).index(c)
		completions.remove(completions.get_iter(index))
	except ValueError:
		pass
	
	completions.append([c])
	if len(completions) >= 30:
		completions.remove(completions.get_iter(0))
	gconf_save_history(list_store_to_list(completions))


gconf_load_history()


#-------------------------------------------------------------------------------


class DeskbarApplet:
	def __init__(self, applet):
		self.gconf_prefix = applet.get_preferences_key()
		self.selected_engine = None
		self.button = None
		self.entry = None
		self.entry_width = -1
		self.engine_names_to_menu_items = {}
		self.show_gray_text = False
		self.set_entry_width_from_gconf(gconf.client_get_default(), None, None, None)

		gconf.client_get_default().notify_add(self.gconf_prefix + "/entry_width", \
			self.set_entry_width_from_gconf)

		en = gconf.client_get_default().get_string(self.gconf_prefix + "/engine_name")
		if (en == None) or (len(en) == 0):
			en = default_engine[1]
		self.set_engine_by_name(en)
		
		self.construct_gui(applet)


	def construct_gui(self, applet):
		box = gtk.HBox()
		box.set_border_width(0)
		
		self.entry = gtk.Entry()
		self.entry.connect("activate", self.on_entry_activate)
		self.entry.connect("focus-in-event", self.on_entry_focus_in)
		self.entry.connect("focus-out-event", self.on_entry_focus_out)
		self.entry.connect("key-press-event", self.on_entry_key_press)
		box.add(self.entry)
		
		self.set_show_gray_text()
		
		completion = gtk.EntryCompletion()
		self.entry.set_completion(completion)
		global completions
		completion.set_model(completions)
		completion.set_text_column(0)

		button = gtk.Button(stock=gtk.STOCK_FIND)
		button.connect_object("clicked", self.on_entry_activate, self.entry)
		button.connect("button-press-event", self.on_button_press)
		box.add(button)
		
		# TODO - replace this all by a MenuToolButton (new in PyGTK 2.6)
	 	gtk.rc_parse_string('''
			style "deskbar-applet-menubar-style"
			{
				GtkMenuBar::shadow-type = none
				GtkMenuBar::internal-padding = 0
			}
			class "GtkMenuBar" style "deskbar-applet-menubar-style"''')
		menu_item = gtk.ImageMenuItem("")
		image = gtk.Image()
		image.set_from_file(gnome_datadir + "/pixmaps/mini-commander/history-mini.xpm")
		menu_item.set_image(image)
		menu_item.set_submenu(self.build_menu())
		menubar = gtk.MenuBar()
		menubar.append(menu_item)
		menubar.connect("button-press-event", self.on_button_press)
		# end TODO
		box.add(menubar)
		
		esr = self.entry.size_request()
		bsr = button.size_request()
		request_y = max(esr[1], bsr[1])
		self.entry.set_size_request(self.entry_width, request_y)
		button.set_size_request(bsr[0], request_y)
		
		applet.set_flags(gtk.CAN_FOCUS)
		applet.add(box)
		applet.connect("destroy", self.on_applet_destroy, None)
		applet.set_applet_flags(gnome.applet.EXPAND_MINOR)
		applet.connect("button-press-event", self.on_applet_button_press)
		applet.connect("size-allocate", self.on_applet_size_allocate)
		applet.show_all()
		
		# funky right-click menu
		menuXml = """
		<popup name="button3">
		        <menuitem
		        	name="Deskbar Applet Preferenes Item"
		        	verb="Prefs"
		        	_label="_Preferences"
		        	pixtype="stock"
		        	pixname="gtk-properties"/>
		        <menuitem
		        	name="Deskbar Applet About Item"
		        	verb="About"
		        	_label="_About"
		        	pixtype="stock"
		        	pixname="gnome-stock-about"/>
		</popup>
		"""
		applet.setup_menu(menuXml, [("About", self.on_about), ("Prefs", self.on_prefs)], applet)


	def set_entry_width_from_gconf(self, client, connection_id, entry, args):
		self.set_entry_width(gconf_get_entry_width(self.gconf_prefix))


	def set_entry_width(self, width):
		if self.entry_width == width:
			return
		
		self.entry_width = width
		gconf.client_get_default().set_int(self.gconf_prefix + "/entry_width", width)
		
		if self.entry != None:
			esr = self.entry.size_request()
			self.entry.set_size_request(width, esr[1])


	def set_engine_by_name(self, name):
		for engine in engines:
			(e_shortcut, e_name, e_cmd) = engine
			if name == e_name:
				self.set_engine(engine)
				break


	def do_query(self, entry, query):
		if len(query) == 0:
			return
		
		add_to_completions(query)
		query = query.replace("\"", "\\\"")
		cmd = self.selected_engine[2].replace("*", query)
		if cmd.startswith("http:"):
			cmd = "gnome-open \"" + cmd + "\""
		os.system(cmd + " &")


	def on_applet_destroy(self, widget, arg):
		del widget


	def on_about(self, component, verb, applet):
		icon = gtk.Image()
		icon.set_from_file(gnome_datadir + "/pixmaps/gnome-mini-commander.png")

		fullname = "Deskbar Applet"
		copyright = "Copyright (C) 2004-2005 Nigel Tao"
		description = "A search bar."
		authors = ["Nigel Tao <nigel.tao@myrealbox.com>","","with thanks to", \
			"Calum Benson (webeyes applet)", "Nikos Kouremenos"]

		about = gnome.ui.About(fullname, VERSION, copyright, description, authors, None, None, icon.get_pixbuf())
		about.set_icon(icon.get_pixbuf())
		about.show()


	def on_entry_activate(self, widget):
		if not self.show_gray_text:
			t = widget.get_text()
			widget.set_text("")
			self.do_query(widget, t)
		
		return gtk.FALSE


	def on_entry_focus_in(self, widget, event):
		if self.show_gray_text:
			self.show_gray_text = False
			widget.set_text("")
			widget.modify_text(gtk.STATE_NORMAL, None)
		
		widget.select_region(0, -1)
		return gtk.FALSE
	
	
	def on_entry_focus_out(self, widget, event):
		t = widget.get_text().strip()
		if len(t) == 0 or t == self.selected_engine[1]:
			self.set_show_gray_text()
		
		return gtk.FALSE


	def set_show_gray_text(self):
		self.show_gray_text = True
		self.entry.set_text(self.selected_engine[1])
		self.entry.select_region(0, 0)
		self.entry.modify_text(gtk.STATE_NORMAL, gray)


	def on_button_press(self, widget, event):
		# allow Middle- and Right-Mouse-Button to go through to the applet window
		if event.button != 1:
			widget.emit_stop_by_name("button-press-event")
		
		return gtk.FALSE


	def on_entry_key_press(self, entry, event):
		# bind Escape to clear the GtkEntry
		if event.keyval == gtk.keysyms.Escape:
			entry.set_text("")

		if event.get_state() & gtk.gdk.CONTROL_MASK:
			for engine in engines:
				(shortcut, name, cmd) = engine
				if shortcut != None and event.keyval == ord(shortcut):
					self.set_engine(engine)
					return gtk.TRUE
		
		return gtk.FALSE


	def build_menu(self):
		accel_group = gtk.AccelGroup()
		menu = gtk.Menu()
		group = None
		for engine in engines:
			(shortcut, name, cmd) = engine
			if cmd == None:
				menu_item = gtk.SeparatorMenuItem()
			else:
				menu_item = gtk.RadioMenuItem(group, name)
				menu_item.connect("activate", self.on_engine_menu_item_activate, engine)
				menu_item.add_accelerator("activate", accel_group, ord(shortcut), \
						gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
				if group == None:
					group = menu_item
				if (self.selected_engine != None) and (name == self.selected_engine[1]):
					menu_item.set_active(True)
				self.engine_names_to_menu_items[name] = menu_item
			menu.append(menu_item)
		menu.show_all()
		return menu


	def on_engine_menu_item_activate(self, widget, engine):
		if widget.get_active():
			self.set_engine(engine)


	def set_engine(self, engine):
		if self.selected_engine == engine:
			return
		
		if engine == None:
			return
		
		try:
			engine_name = engine[1]
			self.engine_names_to_menu_items[engine_name].set_active(True)
		except KeyError:
			pass
		
		gconf.client_get_default().set_string(self.gconf_prefix + "/engine_name", engine[1])
		
		self.selected_engine = engine
		if self.entry != None:
			self.show_gray_text = False
			self.entry.set_text(engine[1])
			self.entry.modify_text(gtk.STATE_NORMAL, None)
			self.entry.select_region(0, -1)
			self.entry.grab_focus()


	def on_applet_button_press(self, widget, event):
		# Left-Mouse-Button should focus the GtkEntry widget (for Fitt's Law
		# - so that a click on applet border on edge of screen activates the
		# most important widget).
		if event.button == 1:
			self.entry.grab_focus()
			return gtk.TRUE
		
		return gtk.FALSE


	def on_applet_size_allocate(self, widget, rect):
		self.entry.set_size_request(self.entry.size_request()[0], rect.height)
		return gtk.FALSE


	def on_prefs(self, component, verb, applet):
		prefs_dialog = PrefsDialog(self.gconf_prefix)
		prefs_dialog.show()
		prefs_dialog.run()
		prefs_dialog.hide()


#-------------------------------------------------------------------------------


class PrefsDialog(gtk.Dialog):
	def __init__(self, gconf_prefix):
		gtk.Dialog.__init__(self, title=("Deskbar Applet Preferences"), \
			buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
		
		self.gconf_prefix = gconf_prefix
		self.set_border_width(5)
		self.set_has_separator(gtk.FALSE)
		
		w = gconf_get_entry_width(gconf_prefix)
		
		adjustment = gtk.Adjustment(w, 0, 4096, 10, 10, 10)
		spinbutton = gtk.SpinButton(adjustment)
		spinbutton.connect("value-changed", self.on_spinbutton)
		
		label_size = gtk.Label("<b>Size</b>")
		label_size.set_use_markup(gtk.TRUE)
		label_size.set_alignment(0.0, 0.5)
		
		width_box_1 = gtk.HBox()
		width_box_1.set_spacing(6)
		width_box_1.add(spinbutton)
		width_box_1.add(gtk.Label("pixels"))
		
		width_box_2 = gtk.HBox()
		width_box_2.set_spacing(12)
		width_box_2.add(gtk.Label("Width:"))
		width_box_2.add(width_box_1)
		
		width_box_3 = gtk.HBox()
		width_box_3.set_spacing(0)
		width_box_3.add(gtk.Label("    "))
		width_box_3.add(width_box_2)
		
		vbox = gtk.VBox()
		vbox.add(label_size)
		vbox.add(width_box_3)
		vbox.set_spacing(6)
		vbox.set_border_width(5)

		self.vbox.pack_start(vbox)

		vbox.show_all()


	def on_spinbutton(self, spinbutton):
		gconf_set_entry_width(self.gconf_prefix, spinbutton.get_value())


#-------------------------------------------------------------------------------


def applet_factory(applet, iid):
	da = DeskbarApplet(applet)
	return gtk.TRUE


if __name__ == "__main__":
	if len(sys.argv) == 2:
		if sys.argv[1] == "-print":
			print "No problems so far."

		elif sys.argv[1] == "-window":
			main_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
			main_window.set_title("Deskbar Applet")
			main_window.connect("destroy", gtk.main_quit)
			
			applet = gnome.applet.Applet()
			applet_factory(applet, None)
			applet.reparent(main_window)
			
			main_window.show_all()
			gtk.main()

	else:
		gnome.applet.bonobo_factory( \
			"OAFIID:Deskbar_Applet_Factory", \
			gnome.applet.Applet.__gtype__, \
			"deskbar_applet", \
			"0", \
			applet_factory)
